rm(list=ls())

library(data.table)
library(xts)
library(viridis)
library(fPortfolio)

### read data 
defs=fread("data/defs.txt",dec=".",sep="\t") # definitions of series
prices=fread("data/prices.txt",dec=".",sep="\t") # read prices 
prices=xts(prices[,-1],order.by=as.Date(prices$date)) # convert them to time series
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE) # calculate returns
returns=timeSeries(returns)

### read data 
allreturns=returns
returns=returns[,c("Esp500","Eftse","Epacific","BtreasLong","BtreasShort","RealEstate")]

### min variance, long only
spec=portfolioSpec()
portf=minvariancePortfolio(data=returns,spec=spec,constraints="LongOnly")
weightsPie(portf)
minrisk=portf
portf

### tangency portfolio
spec=portfolioSpec()
setRiskFreeRate(spec)=0
portf=tangencyPortfolio(data=returns,spec=spec,constraints="LongOnly")
weightsPie(portf)
portf

### frontier, long only
spec=portfolioSpec()
setNFrontierPoints=20
longFrontier=portfolioFrontier(returns,spec)
plot(longFrontier)
tailoredFrontierPlot(longFrontier)

### box constrained, long only
spec=portfolioSpec()
boxc=c("minW[1:6]=0","maxW[1:6]=0.5")
boxfr=portfolioFrontier(returns,spec=spec,constraints=boxc)
tailoredFrontierPlot(boxfr)
portf=minvariancePortfolio(data=returns,spec=spec,constraints=boxc)
weightsPie(portf)

### frontier, all ETF's
spec=portfolioSpec()
setNFrontierPoints=207

longFrontier=portfolioFrontier(returns,spec)
weightsPlot(longFrontier,col=viridis(6))
plot(longFrontier)

############ robust portfolios
covMcdSpec = portfolioSpec()
setEstimator(covMcdSpec) = "covMcdEstimator"
setNFrontierPoints(covMcdSpec) =20
covMcdFrontier= portfolioFrontier(data = returns, spec = covMcdSpec)
print(covMcdFrontier)
tailoredFrontierPlot(covMcdFrontier)

mveSpec = portfolioSpec()
setEstimator(mveSpec) = "mveEstimator"
setNFrontierPoints(mveSpec) =20
mveFrontier= portfolioFrontier(data = returns, spec = mveSpec)
tailoredFrontierPlot(mveFrontier)

ogkSpec = portfolioSpec()
setEstimator(ogkSpec) = "covOGKEstimator"
setNFrontierPoints(ogkSpec) =20
ogkFrontier= portfolioFrontier(data = returns, spec = ogkSpec)
tailoredFrontierPlot(ogkFrontier)

shrinkSpec = portfolioSpec()
setEstimator(ogkSpec) = "shrinkEstimator"
setNFrontierPoints(shrinkSpec) =20
shrinkFrontier= portfolioFrontier(data = returns, spec = shrinkSpec)
tailoredFrontierPlot(shrinkFrontier)

par(mfrow=c(2,2))
weightsPlot(covMcdFrontier,col=viridis(6))
weightsPlot(mveFrontier,col=viridis(6))
weightsPlot(ogkFrontier,col=viridis(6))
weightsPlot(shrinkFrontier,col=viridis(6))

####### CVaR
minriskSpec <- portfolioSpec()
setType(minriskSpec) <- "CVaR"
setAlpha(minriskSpec) <- 0.05
setSolver(minriskSpec) <- "solveRglpk.CVAR"
setTargetReturn(minriskSpec) <- mean(returns)
minriskPortfolio <- efficientPortfolio(data = returns, spec = minriskSpec,constraints = "LongOnly")
weightsPie(minriskPortfolio)

longSpec <- portfolioSpec()
setType(longSpec) <- "CVaR"
setAlpha(longSpec) <- 0.05
setNFrontierPoints(longSpec) <- 20
setSolver(longSpec) <- "solveRglpk.CVAR"
longFrontier <- portfolioFrontier(data = returns, spec = longSpec,constraints = "LongOnly")
tailoredFrontierPlot(object = longFrontier, mText = "Mean-CVaR Portfolio - Long Only Constraints",risk = "CVaR")

##### backtesting
backtSpec <- portfolioSpec()
backtConstraints <- "LongOnly"
backtBacktest <- portfolioBacktest()
setWindowsHorizon(backtBacktest) <- "18m"
setSmootherLambda(backtBacktest) <- "6m"
backtFormula <- Esp500 ~ Eftse + Epacific + BtreasLong + BtreasShort + RealEstate 
backtPortfolios <- portfolioBacktesting(formula = backtFormula,
                                        data = returns, spec = backtSpec, constraints = backtConstraints,
                                        backtest = backtBacktest, trace = FALSE)
setSmootherInitialWeights(backtPortfolios$backtest) <- rep(1/5,5)
setSmootherLambda(backtPortfolios$backtest) <- "12m"
backtSmooth <- portfolioSmoothing(backtPortfolios)
backtestPlot(backtSmooth)

###### network portfolios
library(igraph)
par(mfrow=c(1,1),mar=c(0,0,0,0))
set.seed(10)
returns=returns[,c("EconsDiscr","EconsStapl","Eenergy","Efinancials","Einfo","Ehealth","Eindus","Ematerial","ErealEstate","Eutils")]
mcor=cor(returns)
corrplot(mcor,method="color",type="upper",addCoef.col ="black",diag=F)
mdist=sqrt(2*(1-mcor))
mdist
gfull=graph_from_adjacency_matrix(mdist,mode = "undirected",weighted = T,diag = F)
plot(gfull)
gmst=minimum.spanning.tree(gfull)
plot(gmst)

centr=degree(gmst)
plot(gmst,vertex.size=10*centr)

n=ncol(returns)
covm=cov(returns)

dvec=matrix(0,nrow=n,ncol=1)  # dvec=matrix(0,nrow=n,ncol=1)
Dmat=covm
Amat=matrix(1,nrow=n,ncol=1)  # Amat=matrix(1,nrow=n,ncol=1)
Amat=cbind(Amat,diag(1,n))
Amat=cbind(Amat,centr)
bvec=c(1,rep(0,n),median(centr))
sol=solve.QP(Dmat,dvec,Amat,bvec,meq=1)
model1=sol$solution
model1
