rm(list=ls())

library(xts)
library(NetworkToolbox)
library(igraph)
library(ggplot2)
library(qgraph)

### read data 
defs=fread("data/defs.txt",dec=".",sep="\t") # definitions of series
prices=fread("data/prices.txt",dec=".",sep="\t") # read prices 
prices=xts(prices[,-1],order.by=as.Date(prices$date)) # convert them to time series
returns=diff(prices,lag=1,log=TRUE,na.pad=FALSE) # calculate returns

#calculate correlations
corrmatrix <- cor(returns, use = "pairwise.complete.obs")

#visualize correlation matrix
corrplot(corrmatrix,method="color",type="upper",diag=F)

#full network - complete graph
hcentrality <- hybrid(corrmatrix, BC = "random", beta = 0.1)
qgraph(corrmatrix, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#threshold network (parameter a sets a threshold level)
resThres<-threshold(corrmatrix, thresh = "alpha", a=0.05, normal = FALSE)
hcentrality <- hybrid(resThres$A, BC = "random", beta = 0.1)
qgraph(resThres$A, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#threshold network (proportional method keeps strongest correlations within density)
resThres<-threshold(corrmatrix, thresh = "proportional", normal = FALSE)
hcentrality <- hybrid(resThres$A, BC = "random", beta = 0.1)
qgraph(resThres$A, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#Minimum Spanning Tree
#transofming correlations into distances
mdist <- sqrt(2*(1-corrmatrix))
#create a full graph from the adjacency matrix of distances
gfull <- graph_from_adjacency_matrix(mdist,mode = "undirected",weighted = T,diag = F)
gmst <- minimum.spanning.tree(gfull)
resMST <- get.adjacency(gmst)
hcentrality <- hybrid(resMST, BC = "random", beta = 0.1)
qgraph(resMST, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#Maximum Spanning Tree
resMaST <- MaST(data = corrmatrix, normal = FALSE, depend = FALSE)
hcentrality <- hybrid(resMaST, BC = "random", beta = 0.1)
qgraph(resMaST, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#Triangulated Maximally Filtered Graph
resTMFG <- TMFG(data = corrmatrix, normal = FALSE, depend = FALSE)
hcentrality <- hybrid(resTMFG$A, BC = "random", beta = 0.1)
qgraph(resTMFG$A, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)

#Maximally Filtered Clique Forest
resMFCF <- MFCF(data = corrmatrix, time.series = TRUE, gain.fxn = "logLik", cases = cases,
                min_size = 3, max_size = 10, pval = 0.10, drop_sep = TRUE,
                use_returns = FALSE, na.data = "fiml")
hcentrality <- hybrid(resMFCF$A, BC = "standard", beta = 0.1)
qgraph(resMFCF$A, layout = "spring", groups = defs$class, palette = "ggplot2", label.prop = 1, 
       vsize = 5*hcentrality)
